//-----Controls-----
// 1-9: Set destination XBee address
// 0:   Set broadcast XBee address
// e:   Echo devices
// i:   Add Arduino devices
// ' ': Toggle device updates
// click: Toggle mouse XY control
//------------------

//package com.fieldfx.devicehub;

import processing.core.PApplet;
import xbee.*;
import oscP5.*;
import netP5.*;

public class DeviceHub {
  
  // Creates a dummy PApplet for the parent
/*  public DeviceHub() {
    this(  );
  }*/
  
  public DeviceHub( PApplet parent ) {
    this.parent = parent;
    oscP5 = new OscP5( this, oscListenPort );
    
    // Set up Serialization
    xmlSerializer = new XMLSerializer( parent );
    xmlSerializer.registerType( new Pin() );
    xmlSerializer.registerType( new Device() );
    xmlSerializer.registerType( new OscMap() );
    xmlSerializer.registerType( new DeviceConfig() );
  }
  
  //-----Settings-----
  int            oscListenPort        = 8000;
  int            oscSendPort          = 16000;
  String         oscConnectPattern    = "/1/push4";
  String         oscDisconnectPattern = "/server/disconnect";
  
  //-----Variables-----
  ArduinoControl arduinoControl;
  DeviceConfig   deviceConfig         = new DeviceConfig();
  OscP5          oscP5;
  NetAddressList netAddressList       = new NetAddressList();
  XMLSerializer  xmlSerializer;
  PApplet        parent;
  
  //-------------------
  
  void oscConnect ( String ipAddress ) {
    if( !netAddressList.contains( ipAddress, oscSendPort ) ) {
      netAddressList.add( new NetAddress( ipAddress, oscSendPort ) );
      System.out.print( "### adding " + ipAddress + " to the list." );
    } else {
      System.out.print( "### " + ipAddress + " is already connected." );
    }
    System.out.print( "### currently there are " + netAddressList.list().size() + " remote locations connected." );
  }
  
  void oscDisconnect ( String ipAddress ) {
    if( netAddressList.contains( ipAddress, oscSendPort ) ) {
      netAddressList.remove( ipAddress, oscSendPort );
      System.out.print( "### removing " + ipAddress + " from the list." );
    } else {
      System.out.print( "### " + ipAddress + " is not connected." );
    }
    System.out.print( "### currently there are " + netAddressList.list().size() );
  }
  
  void oscEvent ( OscMessage oscMsg ) {
    if( oscMsg.addrPattern().equals( oscConnectPattern ) ) oscConnect( oscMsg.netAddress().address() );
    else if( oscMsg.addrPattern().equals( oscDisconnectPattern ) ) oscDisconnect( oscMsg.netAddress().address() );
    else //echo back
    oscP5.send( oscMsg, netAddressList );
    
    // Try to change a the label
    OscMessage oscMsgToSend = new OscMessage("/1/label1");
    oscMsgToSend.add("poo");
    oscP5.send( oscMsgToSend, netAddressList );
    
    System.out.println( "received osc msg: " + oscMsg.addrPattern() + " " + oscMsg.typetag() );
  }
  
  public void createDefaultConfig ( String fileName ) {
    deviceConfig.addDevice( new Device( "xyServo", 1, new int[] { 9, 10 } ) );
    deviceConfig.addDevice( new Device( "xyServo", 1, new int[] { 11, 12 } ) );
    deviceConfig.addDevice( new Device( "xyServo", 2, new int[] { 9, 10 } ) );
    deviceConfig.addOscMap( new OscMap( "laser1", "/multi/1", "setXY" ) );
    deviceConfig.addOscMap( new OscMap( "laser2", "/multi/2", "setXY" ) );
    xmlSerializer.save( "deviceConfig.xml", deviceConfig );
  }
  
  public Device findDevice ( String deviceName ) {
    if( deviceConfig != null ) return deviceConfig.findDevice( deviceName );
    else return null;
  }
  
  void xBeeEvent ( XBeeReader xbee ) {
    if( arduinoControl != null ) arduinoControl.Read();
  }
  
  public void connect ( String configFileName ) {
    if( arduinoControl == null ) {
      arduinoControl = new ArduinoControl();
			if(arduinoControl != null)
      	arduinoControl.Setup( parent );
      xmlSerializer.load( configFileName, deviceConfig );
      deviceConfig.initDevices();
      deviceConfig.initOscMaps( oscP5 );
    }
  }
  
  public void update () {
    connect( parent.sketchPath("deviceConfig.xml") );
    arduinoControl.Read();
    
    deviceConfig.sendAll( arduinoControl );
  }
}
