//-----Controls-----
// 1-9: Set destination XBee address
// 0:   Set broadcast XBee address
// e:   Echo devices
// i:   Add Arduino devices
// ' ': Toggle device updates
// click: Toggle mouse XY control
//------------------


import xbee.*;
import oscP5.*;
import netP5.*;
import processing.serial.*;
import java.util.ArrayList;
import java.util.List;
import javax.sound.midi.*;

import com.fieldfx.devicehub.*;
import com.fieldfx.util.serialize.*;

DeviceHub                        deviceHub;
javax.sound.midi.Sequencer       midiSequencer;
com.fieldfx.devicehub.Sequencer  sequencer;

//this is useful
int[]       testSeq = new int[] {
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31
};
int         switchIdx = 0;
int         sweepWidth = 1;
int         patternIdx = 2;// 0 == none
int         sequenceIdx = 0;
int         animDelayMillis = 100;
List<int[]> sequences = new ArrayList<int[]>();
int[]       bakedSequence = null;
int         bakedSeqIdx = 0;
int         bakedSeqDelayMillis = 0;// 0 == auto
int         numPatterns = 2;


// ---------------------------------------------------------------------------------------------------------------- //
void setup() {
  frameRate(99999);
  deviceHub = new DeviceHub(this);
  sequencer = new com.fieldfx.devicehub.Sequencer();
  deviceHub.oscP5.plug( this, "fader1Test",     "/1/fader1" );
  deviceHub.oscP5.plug( this, "fader2Test",     "/1/fader2" );
  deviceHub.oscP5.plug( this, "resetSequencer", "/1/push10" );
  deviceHub.oscP5.plug( this, "incSwitchIdx",   "/1/push11" );
  deviceHub.oscP5.plug( this, "decSwitchIdx",   "/1/push12" );
  deviceHub.oscP5.plug( this, "incPatternIdx",  "/1/push1"  );
  deviceHub.oscP5.plug( this, "incSequenceIdx", "/1/push2"  );
  setupSequences();
  setupMidi();
}
// ---------------------------------------------------------------------------------------------------------------- //
void draw() {
  if (deviceHub.arduinoControl == null) {
    deviceHub.connect("deviceConfig.xml");
    sequencer.reset(sequences.get(sequenceIdx), deviceHub.findDevice("switches2"), deviceHub.findDevice("switches1"));
  }

  if (patternIdx == 1) {
    ++switchIdx;
    updateSequencer();
    delay(animDelayMillis);
  }
  else if (patternIdx == 2) {
    int switchValues = bakedSequence[bakedSeqIdx];
    updateSequencer();
    bakedSeqIdx = (bakedSeqIdx + 1) % bakedSequence.length;
    delay(bakedSeqDelayMillis);
  }
}


// ---------------------------------------------------------------------------------------------------------------- //
int lastUpdateMillis = 0;
int minUpdateMillis = 25;
void updateSequencer() {
  int millisSinceLastUpdate = millis() - lastUpdateMillis;
  if (millisSinceLastUpdate < minUpdateMillis)
    return;

  // Update the sequence, based on patternIdx
  if (patternIdx == 2)
    sequencer.updateBakedSequence(bakedSequence[bakedSeqIdx]);
  else
    sequencer.updateSweep(switchIdx, sweepWidth);

  if (sequencer.needsUpdate) {
    deviceHub.update();
    sequencer.needsUpdate = false;
    lastUpdateMillis = millis();
  }
}

// ---------------------------------------------------------------------------------------------------------------- //
void fader1Test(float val) {
  if (patternIdx == 0)
    switchIdx = (int)map(val, 0, 1, 0, sequences.get(sequenceIdx).length-1);
  else
    animDelayMillis = (int)map(val, 0, 1, 1000, minUpdateMillis);
  updateSequencer();
}

// ---------------------------------------------------------------------------------------------------------------- //
void fader2Test(float val) {
  sweepWidth = (int)map(val, 0, 1, 1, sequences.get(sequenceIdx).length-1);
  updateSequencer();
}

// ---------------------------------------------------------------------------------------------------------------- //
void resetSequencer(float val) {
  if (val == 0.f)
    return;
  switchIdx = 0;
  patternIdx = 0;
  updateSequencer();
}

// ---------------------------------------------------------------------------------------------------------------- //
void incSwitchIdx(float val) {
  if (val == 0.f)
    return;
  ++switchIdx;
  updateSequencer();
}
// ---------------------------------------------------------------------------------------------------------------- //
void decSwitchIdx(float val) {
  if (val == 0.f)
    return;
  --switchIdx;
  updateSequencer();
}
// ---------------------------------------------------------------------------------------------------------------- //
void incPatternIdx(float val) {
  if (val == 0.f)
    return;
  ++patternIdx;
  patternIdx %= numPatterns+1;
  //println("patternIdx="+patternIdx);
  updateSequencer();
}
// ---------------------------------------------------------------------------------------------------------------- //
void incSequenceIdx(float val) {
  if (val == 0.f)
    return;
  sequenceIdx = (sequenceIdx + 1) % sequences.size();
  sequencer.reset(sequences.get(sequenceIdx), deviceHub.findDevice("switches2"), deviceHub.findDevice("switches1"));
  updateSequencer();
}
// ---------------------------------------------------------------------------------------------------------------- //
void toggleSwitches(String deviceName) {
  Device switchArray1 = deviceHub.findDevice(deviceName);
  if (switchArray1 != null) {
    boolean curVal = switchArray1.getSwitch(0);
    switchArray1.setSwitchArray( !curVal );
  }
}
// ---------------------------------------------------------------------------------------------------------------- //
void keyPressed() {
  if (key == ' ') {
    toggleSwitches("switches1");
    toggleSwitches("switches2");
  }
  else if (key == 'c') {
    deviceHub.arduinoControl.SetDestAddress(0);
    deviceHub.arduinoControl.ClearCmdBuffer();
  }
  else if (key == 'i') { // Init
    deviceHub.deviceConfig.sendDevices(deviceHub.arduinoControl);
  }
  else if (key == 's') { // Save to EEPROM
    deviceHub.deviceConfig.sendAll(deviceHub.arduinoControl);
    deviceHub.arduinoControl.SetDestAddress(0);
    deviceHub.arduinoControl.SaveCmdBuffer();
  }
  else if (key == 'l') { // Load 
    deviceHub.arduinoControl.SetDestAddress(0);
    deviceHub.arduinoControl.LoadCmdBuffer();
  }
  else if (key == 'x') { // Execute
    deviceHub.arduinoControl.SetDestAddress(0);
    deviceHub.arduinoControl.ExecuteCmdBuffer();
  }
  else if (key == 'e')
    deviceHub.arduinoControl.EchoDevices();

  deviceHub.update();
}

// ---------------------------------------------------------------------------------------------------------------- //
void oscEvent ( OscMessage oscMsg ) {
  deviceHub.oscEvent(oscMsg);
}

// ---------------------------------------------------------------------------------------------------------------- //
void setupSequences() {
  sequences.add(new int[] {
    9, 18, 16, 27, 2, 21, 24
  }
  );
  sequences.add(new int[] {
    7, 25, 0, 10, 14, 9, 15
  }
  );
  sequences.add(new int[] {
    18, 8, 6, 6, 8
  }
  );
  sequences.add(testSeq);
}
// ---------------------------------------------------------------------------------------------------------------- //
int updateLightFlags( int currentLights, int lightIndex, boolean on ) {
  if ( lightIndex >= 32 )
    return currentLights;

  int  bitIdx   = lightIndex % 32;
  int  byteMask = (int)( 1 << ( 32 - bitIdx - 1 ) );

  if ( on )
    currentLights |= byteMask;
  else
    currentLights &= ~byteMask;

  // println(currentLights);
  return currentLights;
}
// ---------------------------------------------------------------------------------------------------------------- //
void setupMidi() {
  try {
    File midiFile = new File(sketchPath("../LightHubEditor/data/xmas_mix4.mid"));
    midiSequencer = MidiSystem.getSequencer();
    midiSequencer.open();
    Sequence sequence = MidiSystem.getSequence(midiFile);

    Track[] tracks    = sequence.getTracks();
    int divisor = 48;
    long tickLength   = sequence.getTickLength() / divisor;
    if (bakedSeqDelayMillis == 0)
      bakedSeqDelayMillis = (int)(sequence.getMicrosecondLength()/1000/tickLength);
    //int  ticksPerBeat = tracks[0].getTimingResolution();

    // Figure out how many measures
    // allocate array of ints at 64*measures
    // Bake the changing bits into the array of integers 
    // int   currentLights = 0x00;
    bakedSequence = new int[(int)tickLength + 1];

    long   lastTick      = 0;
    for ( int i=0; i<bakedSequence.length; ++i )
      bakedSequence[i] = 0x00;

    for ( int trackIdx=0; trackIdx<tracks.length; ++trackIdx ) {

      //currentLights = bakedSequence[0];
      //int trackIdx = 1;
      lastTick   = 0;
      int eventCount = tracks[trackIdx].size();

      boolean first = true;
      int parseSongPointer = 0;
      int songPointer = 0;
      for ( int event=0; event<eventCount; ++event ) {
        MidiEvent    midiEvent  = tracks[trackIdx].get( event );
        MidiMessage  message    = midiEvent.getMessage();
        byte[]       data       = message.getMessage();

        if ( data[0] == -1 && data[1] == 9 ) {
          parseSongPointer = 2;
        } 
        else if ( data[0] == -78 && parseSongPointer > 0 ) {
          if ( parseSongPointer == 1 )
            songPointer <<= 8;
          songPointer += data[1];
          --parseSongPointer;
        } 
        else if ( data[0] == -110 || data[0] == -126 ) {

          int          lightIndex = data[1] - (int)0x3C;
          long         tick       = (songPointer*6 + midiEvent.getTick()) / divisor;

          boolean on = data[0] == -110? false : true;
          for ( long t = lastTick; t <= tick; ++t )
            bakedSequence[(int)t] = updateLightFlags( bakedSequence[(int)t], lightIndex, on );
          lastTick = tick;
        }
      }
    }
  } 
  catch(Exception e) {
    e.printStackTrace();
  }
}

