import java.util.List;

class Sequencer {
	public void reset(int[] switchIndexes, Device switchArray1, Device switchArray2) {
		curIdx = -1;
		prevStartIdx = -1;
		prevSweepWidth = -1;
    prevSwitchMask = 0;
		needsUpdate = true;
		this.switchIndexes = switchIndexes;
		this.switchArray1 = switchArray1;
		this.switchArray2 = switchArray2;
	}
	
	public void updateSweep() {
		setSwitch(curIdx, false);
		incrementIdx();
		setSwitch(curIdx, true);
	}
	
	public void updateSweep(int startIdx, int sweepWidth) {
		// Don't update if we were passed the same values as last time
		if(startIdx == prevStartIdx && sweepWidth == prevSweepWidth) {
			return;
		}
		
		clearSwitches();
		for(int i=0; i<sweepWidth; ++i)
			setSwitch((startIdx+i)%switchIndexes.length, true);
			
		prevStartIdx = startIdx;
		prevSweepWidth = sweepWidth;
		needsUpdate = true;
	}
  
  public void updateBakedSequence(int switchMask) {
		// Don't update if we were passed the same values as last time
		if(prevSwitchMask == switchMask) {
			return;
		}
		
		clearSwitches();
		setSwitchMask(switchMask);
			
		prevSwitchMask = switchMask;
		needsUpdate = true;
	}
	
	void setSwitch(int idx, boolean value) {
		// Check range
		if(idx < 0 || idx >= switchIndexes.length)
			return;
		
		int switchIdx = switchIndexes[idx];
		
		// Ignore invalid switches
		if(switchIdx < 0)
			return;
		
		Device device = (switchIdx < 16 ? switchArray1 : switchArray2);
		if(device != null)
			device.setSwitch(switchIdx % 16, value);
	}
  void setSwitchMask(int mask) {
		//System.out.print(mask+"\n");
		if(switchArray1 != null)
			switchArray1.setSwitchMask(mask & 0xFFFF0000);
    if(switchArray2 != null)
			switchArray2.setSwitchMask(mask << 16);
	}
	
	void clearSwitches() {
		if(switchArray1 != null) switchArray1.setSwitchArray(false);
		if(switchArray2 != null) switchArray2.setSwitchArray(false);
	}
	
	private void incrementIdx() {
		++curIdx;
		curIdx %= switchIndexes.length;
	}
	private void decrementIdx() {
		--curIdx;
		curIdx %= switchIndexes.length;
	}
	
	public boolean needsUpdate = true;
	
	private int[]  switchIndexes = null;
	private Device switchArray1 = null;
	private Device switchArray2 = null;
	private int 	 curIdx = -1;
	private int		 prevStartIdx = -1;
	private int		 prevSweepWidth = -1;
  private int    prevSwitchMask = 0;
}