//import processing.opengl.*;

//import javax.sound.midi.spi.MidiFileWriter;
import javax.sound.midi.*;
import java.io.*;

import java.util.List;
import java.util.ArrayList;
import java.util.TreeSet;

import com.fieldfx.math.*;
import com.fieldfx.util.serialize.*;

// Wizzard / config mode iterates through list of light channels, a light set or none is selected, then next one is done.
// Serialize this data so it only needs to be done once.

EditState       state               = new EditState();
LightSet        currentSet          = null;

Vector2         selected            = null;
PFont           font;
int             setIndex            = 0;
int             lightIndex          = 0;


// ------------------------------------------------------------------------------------------ //
void setup() {
  size( screen.width-100, screen.height-100, P3D );
  load();
  font = createFont("Ariel",14);
}

// ------------------------------------------------------------------------------------------ //
void draw() {
  background(0);

  if( state.currentPath != null &&
      state.editMode != state.config  )
      state.currentPath.draw();
  
  strokeWeight( 1 );
  stroke(128);
  for( LightSet set : state.sets ) {
    set.draw();
  }
  
  if( currentSet != null ) {
    strokeWeight( 2 );
    stroke(255);
    currentSet.draw();
  }
  
  
  if( state.editMode != state.config ) {
    strokeWeight( 2 );
    stroke(255);
    if( state.currentPath != null && state.currentPath.locations != null ) {
      List<IntersectionPoint> points = state.currentPath.locations.points;
      if( points.size() > 0 ) {
        int index   = 0;
        int skipIdx = (frameCount / 5) % points.size();
        stroke(255);
        for( IntersectionPoint point : points ) {
          if( skipIdx == index )
            state.sets.get( point.segmentIndex ).draw();
          ++index;
        }
      }
    }
    
  } else {
    for( int i=0; i<32; ++i ) {
      int y = (int)map( i, 0,32, 30, height-30 );
      
      noFill();
      stroke(255);
      if( i == lightIndex )
        rect( width - 80, y - 16, 70, 20 );
        
      fill( 255 );
      text( i, width - 40, y );
    }
  }
}

// ------------------------------------------------------------------------------------------ //
void save() {
  File toSave = new File( sketchPath("bnl.xml") );
  if( toSave.exists() ) {
    toSave.delete();
  }
  
  XMLSerializer saver = new XMLSerializer( this );
                saver.save( toSave.getAbsolutePath(), state );
}
// ------------------------------------------------------------------------------------------ //
void load() {
  File toLoad = new File( sketchPath("bnl.xml") );
  if( toLoad.exists() ) {
    XMLSerializer loader = new XMLSerializer( this );
                  loader.registerType( new LightSet() );
                  loader.registerType( new Vector2() );
                  loader.registerType( new EditState() );
                  loader.registerType( new SequencePath() );
                  loader.load( toLoad.getAbsolutePath(), state );
  }
}

// ------------------------------------------------------------------------------------------ //
void addSet() {
  currentSet = new LightSet();
  state.sets.add( currentSet );
  println("New set added.");
}
// ------------------------------------------------------------------------------------------ //
void addLight( Vector2 where ) {
  if( currentSet != null ) {
    currentSet.lights.add( where );
    println( "Light added at " + where.toString() );
  }
}
// ------------------------------------------------------------------------------------------ //
void addPath( Vector2 where ) {
  state.currentPath = new SequencePath( where );
}

// ------------------------------------------------------------------------------------------ //
void nextSet    () { 
  setIndex   = (++setIndex + state.sets.size()) % state.sets.size();
  currentSet = state.sets.get( setIndex );
  lightIndex = currentSet.lightIndex;
}
// ------------------------------------------------------------------------------------------ //
void prevSet    () {
  setIndex   = (--setIndex + state.sets.size()) % state.sets.size();
  currentSet = state.sets.get( setIndex );
  lightIndex = currentSet.lightIndex;
}
// ------------------------------------------------------------------------------------------ //
void printSequenceArray() {
  state.currentPath.printSequenceArray();
}
// ------------------------------------------------------------------------------------------ //
void saveSequenceMidi() {
  state.currentPath.saveSequenceMidi(12);
}

// ------------------------------------------------------------------------------------------ //
void keyPressed() {
  if( state == null ) return;
  
  if( key == 's' || key == 'S' ) save();
  if( key == 'l' || key == 'L' ) load();
  if( key == '1' ) { println("Path mode.");     state.editMode = state.place;    }
  if( key == '2' ) { println("Movement mode."); state.editMode = state.move;     }
  if( key == '3' ) { println("Sequence mode."); state.editMode = state.sequence; }
  if( key == '4' ) { println("Config mode.");   state.editMode = state.config;   }


  if( state.editMode == state.place ) {
    if( key == ' ' ) addSet();
  } else if( state.editMode == state.sequence ) {
    if( key == ' ' ) printSequenceArray();
    if( key == 's' || key == 'S' ) saveSequenceMidi();
  } else if( state.editMode == state.config ) {
    if( keyCode == LEFT  ) nextSet();
    if( keyCode == RIGHT ) prevSet();
  }
}
// ------------------------------------------------------------------------------------------ //
void mousePressed() {
  Vector2 where = new Vector2( mouseX, mouseY );
  if( state.editMode == state.place ) {
    if( mouseButton == LEFT ) {
      selected = where;
      addLight( where );
    }
  } else if( state.editMode == state.move ) {
    selected = state.findClosest( where );
  } else if( state.editMode == state.sequence ) {
    addPath( where );
  } if( state.editMode == state.config ) {
    if( mouseX > (width - 80) ) {
      lightIndex = (int)map( mouseY, 30, height-30, 0,32 );
      lightIndex %= 32;
      if( currentSet != null )
        currentSet.lightIndex = lightIndex;
    }
  }
}
// ------------------------------------------------------------------------------------------ //
void mouseReleased() {
  if( state.editMode == state.place ) {
    selected = null;
  } else if( state.editMode == state.move ) {
    selected = null;
  }
}
// ------------------------------------------------------------------------------------------ //
void mouseDragged() {
  if( selected != null ) {
    selected.set( mouseX, mouseY );
  }
}
void mouseMoved() {
  if( state.editMode == state.sequence ) {
    if( state.currentPath != null )
      state.currentPath.end.set(mouseX,mouseY);
  }
}

