
class LightSet extends SerializableBase {
  int             lineWeight = 2;
  color           lightColor = color(255);
  int             lightIndex = 0;
  List< Vector2 > lights     = new ArrayList< Vector2 >();
  
  // ------------------------------------------------------------------------------------------ //
  public String       getType   ( )              { return "LightSet"; }
  public Serializable clone     ( )              { return new LightSet(); }
  public void         serialize ( Serializer s ) {
    lineWeight = s.serialize( "lineWeight", lineWeight );
    lightColor = s.serialize( "lightColor", lightColor );
    lightIndex = s.serialize( "lightIndex", lightIndex );
                 s.serialize( "lights",     lights     );
  }
  
  // ------------------------------------------------------------------------------------------ //
  public Vector2 findClosest( Vector2 where ) {
    if( lights != null && lights.size() > 0 ) {
      Vector2 closest        = lights.get(0);
      float   closestDistSqr = closest.sub(where).lenlen();
      for( Vector2 lightVert : lights ) {
        if( lightVert == closest ) continue;
        float distSqr2LightVert = lightVert.sub(where).lenlen();
        if( distSqr2LightVert < closestDistSqr ) {
          closestDistSqr = distSqr2LightVert;
          closest        = lightVert;
        }
      }
      return closest;
    }    
    return null;
  }
  // ------------------------------------------------------------------------------------------ //
  public List<Vector2> collectIntersections( LineSeg2 intersectingLine ) {
    List<Vector2> results = new ArrayList<Vector2>();
    for( int i=0; i<lights.size()-1; ++i ) {
      Vector2  v0      = lights.get( i );
      Vector2  v1      = lights.get( i + 1 );
      LineSeg2 segment = new LineSeg2( v0, v1 );
      Vector2  result  = intersectingLine.intersect( segment );
      
      if( result != null ) {
        results.add( result );
      }
    }
    
    return results;
  }
  // ------------------------------------------------------------------------------------------ //
  public void draw() {
    for( int i=0; i<lights.size()-1; ++i ) {
      Vector2 v0 = lights.get(i);
      Vector2 v1 = lights.get(i+1);
      line(v0.x,v0.y,v1.x,v1.y);
    }
  }
}

