


public class SequencePath implements com.fieldfx.util.serialize.Serializable {
  public Vector2 center = new Vector2();
  public Vector2 end    = new Vector2();
  public Intersections   locations = null;
  
  public SequencePath()                 {  }
  public SequencePath( Vector2 center ) { this(); this.center = center; }
  
  public String       getType   ( )              { return "SequencePath"; }
  public com.fieldfx.util.serialize.Serializable clone     ( )              { return new SequencePath(); }
  public void         serialize ( Serializer s ) {
    s.serialize( "center", center );
    s.serialize( "end",    end    );
  }
  
  
  
  
  public void printSequenceArray() {
    if( locations == null || locations.points == null ) return;
    
    boolean first = true;
    print( "{ " );
    for( IntersectionPoint p : locations.points ) {
      if( !first ) {
        print( ", " );
      }
      first = false;
      LightSet set = state.sets.get( p.segmentIndex );
      print( set.lightIndex );
    }
    print( " }" );
  }
  
  
  private void addEvent( Track mainTrack, int command, int tick, int channel, int data1, int data2 ) {
    try {
      ShortMessage   mm    = new ShortMessage();
                     mm.setMessage( command, channel, data1, data2 );
      MidiEvent      me    = new MidiEvent( mm, tick );
      mainTrack.add( me );
    } catch( Exception e ) {
      e.printStackTrace();
    }
  }
  
  public void saveSequenceMidi( int noteDuration ) {
    if( locations == null || locations.points == null ) return;    
    // Build the sequence
    try {
      //****  Create a new MIDI sequence with 24 ticks per beat  ****
      Sequence s = new Sequence(javax.sound.midi.Sequence.PPQ,24);
  
      //****  Obtain a MIDI track from the sequence  ****
      Track t = s.createTrack();
  
      //****  General MIDI sysex -- turn on General MIDI sound set  ****
      byte[] b = {(byte)0xF0, 0x7E, 0x7F, 0x09, 0x01, (byte)0xF7};
      SysexMessage sm = new SysexMessage();
      sm.setMessage(b, 6);
      MidiEvent me = new MidiEvent(sm,(long)0);
      t.add(me);
  
      //****  set tempo (meta event)  ****
      MetaMessage mt = new MetaMessage();
      byte[] bt = {0x02, (byte)0x00, 0x00};
      mt.setMessage(0x51 ,bt, 3);
      me = new MidiEvent(mt,(long)0);
      t.add(me);
  
      //****  set track name (meta event)  ****
      mt = new MetaMessage();
      String TrackName = new String("BlinkinLights Sequence Track");
      mt.setMessage(0x03 ,TrackName.getBytes(), TrackName.length());
      me = new MidiEvent(mt,(long)0);
      t.add(me);
  
  
  
      int index = 0;
      for( IntersectionPoint p : locations.points ) {
        LightSet set = state.sets.get( p.segmentIndex );
        
        int tick_on  = noteDuration * index;
        int tick_off = tick_on + noteDuration;
        int channel  = 2;
        
        addEvent( t, ShortMessage.NOTE_ON,  tick_on,  channel, 0x3C + set.lightIndex, 0x60 );
        addEvent( t, ShortMessage.NOTE_OFF, tick_off, channel, 0x3C + set.lightIndex, 0x40 );
        
        ++index;
      }
      
      int song_end  = noteDuration * index;
  
  
      //****  set end of track (meta event) 19 ticks later  ****
      mt = new MetaMessage();
      byte[] bet = {}; // empty array
      mt.setMessage(0x2F,bet,0);
      me = new MidiEvent(mt, (long)song_end);
      t.add(me);
  
      //****  write the MIDI sequence to a MIDI file  ****
      File f = new File( sketchPath("out.mid") );
      MidiSystem.write(s,1,f);
    } catch(Exception e) {
      System.out.println("Exception caught " + e.toString());
    } //catch
  }
  
  public void draw() {
    if( center == null ) return; 
    
    textMode( SCREEN );
    textFont( font );
    
    float dx = (end.x - center.x);
    float dy = (end.y - center.y);
    LineSeg2        intersectingLineSeg = new LineSeg2( center.x - dx, center.y - dy, center.x + dx, center.y + dy);
                    locations           = state.collectSetIntersections( intersectingLineSeg, 64 );
                    locations.draw           ( );
                    stroke                   ( 255, 0, 0 );
                    intersectingLineSeg.draw ( g );
  }
}
