package com.fieldfx.devicehub;

import java.util.ArrayList;
import java.util.List;

import com.fieldfx.util.serialize.Serializable;
import com.fieldfx.util.serialize.Serializer;

public class Device implements Serializable {
  //TODO: Refactor
  public int     x           = 0;
  public int     y           = 0;
  public int     z           = 0;
  public char    bytes[]     = new char[4];
  /*  public String setFuncName = "";*/

  private String         deviceType  = "";
  private String         name        = "";
  private int            xbeeAddress = 0;
  private int            deviceIndex = 0;
  private List<Pin>      pins        = new ArrayList<Pin>();

  
  // -------------------------------------------------------------------- //
  public Device() {}
  public Device( String deviceType_, int xbeeAddress_, int[] pins_ ) {
    deviceType = deviceType_;
    xbeeAddress = xbeeAddress_;
    if( pins != null ) for( int pin : pins_ ) {
      pins.add( new Pin( pin ) );
    }
  }
  
  // -------------------------------------------------------------------- //
  @Override
  public String getType () {
    return "Device";
  }
  
  // -------------------------------------------------------------------- //
  @Override
  public Device clone () {
    return new Device();
  }
  
  // -------------------------------------------------------------------- //
  @Override
  public void serialize ( Serializer s ) {
    name = s.serialize( "name", name );
    deviceType = s.serialize( "deviceType", deviceType );
    xbeeAddress = s.serialize( "xbeeAddress", xbeeAddress );
    s.serialize( "pins", pins );
  }
  
  // -------------------------------------------------------------------- //
  public void attach ( ArduinoControl arduinoControl ) {
    System.out.println( "Attaching " + deviceType + " " + name + " on " + xbeeAddress + "." + deviceIndex );
    
    ///////TODO: Refactor to remove this step
    arduinoControl.setDestAddress( xbeeAddress );
    if( deviceType.matches( "xyServo" ) ) {
      arduinoControl.addXYServo( pins.get( 0 ).get(), pins.get( 1 ).get(), pins.get( 2 ).get() );
    } else if( deviceType.matches( "pwm3" ) ) {
      arduinoControl.addPWM3( pins.get( 0 ).get(), pins.get( 1 ).get(), pins.get( 2 ).get() );
    } else if( deviceType.matches( "switchArray" ) ) {
      arduinoControl.addSwitchArray( pins.get( 0 ).get(), pins.get( 1 ).get(), pins.get( 2 ).get() );
    } else if( deviceType.matches( "screen" ) ) {
      arduinoControl.addScreen();
    } else if( deviceType.matches( "cmdBuffer" ) ) {
      arduinoControl.addCmdBuffer( 1 );
    } else System.out.println( "Device::attach() Invalid device type " + deviceType );
  }
  
  // -------------------------------------------------------------------- //
  public void setX ( float x_ ) {
    x = (int)( x_ * 255 );
  }
  
  // -------------------------------------------------------------------- //
  public void setY ( float y_ ) {
    y = (int)( y_ * 255 );
  }
  
  // -------------------------------------------------------------------- //
  public void setZ ( float z_ ) {
    z = (int)( z_ * 255 );
  }
  
  // -------------------------------------------------------------------- //
  public float getX () {
    return (float)x / 255.f;
  }
  
  // -------------------------------------------------------------------- //
  public float getY () {
    return (float)y / 255.f;
  }
  
  // -------------------------------------------------------------------- //
  public float getZ () {
    return (float)z / 255.f;
  }
  
  // -------------------------------------------------------------------- //
  public void setXY ( float x_, float y_ ) {
    x = (int)( x_ * 255 );
    y = (int)( y_ * 255 );
  }
  
  // -------------------------------------------------------------------- //
  public void setXYZ ( float x_, float y_, float z_ ) {
    x = (int)( x_ * 255 );
    y = (int)( y_ * 255 );
    z = (int)( z_ * 255 );
  }
  
  // -------------------------------------------------------------------- //
  public void setSwitchiii ( int x, int y, int z ) {
    if( z != 0 ) return;
    int switchIdx = x * y + x;
    boolean switchValue = !getSwitch( switchIdx );
    setSwitch( switchIdx, switchValue );
  }
  
  // -------------------------------------------------------------------- //
  public void setSwitch ( int switchIdx, boolean value ) {
    int numSwitches = 8 * bytes.length;
    if( switchIdx >= numSwitches )
			return;
		
    int byteIdx = switchIdx / 8;
    int bitIdx = switchIdx % 8;
    char byteMask = (char)( 1 << ( 7 - bitIdx ) );
    if( value == true )
			bytes[byteIdx] |= byteMask;
    else
			bytes[byteIdx] &= ~byteMask;
    System.out.println( "setSwitch(" + switchIdx + ", " + value + "); byte " + byteIdx + ", bit " + bitIdx + ", mask "
        + (int)byteMask );
    System.out.println( "  bytes " + (int)bytes[0] + " " + (int)bytes[1] + " " + (int)bytes[2] + " " + (int)bytes[3] );
  }
  
  // -------------------------------------------------------------------- //
  public void setSwitchMask ( int switchMask ) {
    bytes[0] = (char)(switchMask >> 24);
    bytes[1] = (char)(switchMask >> 16);
    bytes[2] = (char)(switchMask >> 8);
    bytes[3] = (char)(switchMask >> 0);

    System.out.println( "  bytes " + (int)bytes[0] + " " + (int)bytes[1] + " " + (int)bytes[2] + " " + (int)bytes[3] );
  }

  // -------------------------------------------------------------------- //
	public void setSwitchArray ( boolean value ) {
		byte newVal = (byte)(value ? 0xFF : 0);
		for(int i=0; i<bytes.length; ++i)
			bytes[i] = (char)newVal;
  }
  
  // -------------------------------------------------------------------- //
  public boolean getSwitch ( int switchIdx ) {
    int numSwitches = 8 * bytes.length;
    if( switchIdx >= numSwitches ) return false;
    int byteIdx = switchIdx / 8;
    int bitIdx = switchIdx % 8;
    char byteMask = (char)( 1 << ( 7 - bitIdx ) );
    return ( bytes[byteIdx] & byteMask ) != 0;
  }

  
  // -------------------------------------------------------------------- //
  //HACKHACK
  public void setColorHACK ( String str, int r, int g, int b ) {
    x = r;
    y = g;
    z = b;
  }
  
  // -------------------------------------------------------------------- //
  public int getIndex () {
    return deviceIndex;
  }
  
  // -------------------------------------------------------------------- //
  public void setIndex ( int deviceIndex_ ) {
    deviceIndex = deviceIndex_;
  }
  
  // -------------------------------------------------------------------- //
  public String getName () {
    return name;
  }
  
  // -------------------------------------------------------------------- //
  public int getXbeeAddress () {
    return xbeeAddress;
  }
  
  // -------------------------------------------------------------------- //
  public void send ( ArduinoControl arduinoControl ) {
    arduinoControl.setDestAddress( xbeeAddress );
    
    if      ( deviceType.matches( "xyServo"     ) ) arduinoControl.sendPosition    ( x, y, z, deviceIndex );
    else if ( deviceType.matches( "pwm3"        ) ) arduinoControl.sendPosition    ( x, y, z, deviceIndex );
    else if ( deviceType.matches( "switchArray" ) ) arduinoControl.sendSwitchArray ( bytes,   deviceIndex );
  }
  
}
