package com.fieldfx.devicehub;

import java.util.ArrayList;
import java.util.List;

import com.fieldfx.util.serialize.Serializable;
import com.fieldfx.util.serialize.Serializer;

import oscP5.OscP5;

//TODO: Move devices into ArduinoControl
public class DeviceConfig implements Serializable {
  
  // -------------------------------------------------------------------- //
  @Override
  public String getType () {
    return "DeviceConfig";
  }
  // -------------------------------------------------------------------- //
  @Override
  public DeviceConfig clone () {
    return new DeviceConfig();
  }
  // -------------------------------------------------------------------- //
  @Override
  public void serialize ( Serializer s ) {
    s.serialize( "devices", devices );
    s.serialize( "oscMaps", oscMaps );
  }
  
  // -------------------------------------------------------------------- //
  public void addDevice ( Device device ) {
    devices.add( device );
  }
  
  // -------------------------------------------------------------------- //
  public void initDevices () {
    int[] deviceCounts = new int[32];
    // Add the device to the Arduino
    for( Device device : devices ) {
      device.setIndex( deviceCounts[device.getXbeeAddress()]++ );
    }
  }
  
  // -------------------------------------------------------------------- //
  public void sendDevices ( ArduinoControl arduinoControl ) {
    // Add the device to the Arduino
    for( Device device : devices ) {
      //device.setIndex( device.getIndex() + 1 ); //add 1 for the DeviceConfig that's not saved
      device.attach( arduinoControl );
    }
  }
  
  // -------------------------------------------------------------------- //
  public Device findDevice ( String deviceName ) {
    for( Device device : devices ) {
      if( device.getName().matches( deviceName ) ) return device;
    }
    return null; //not found
  }
  
  // -------------------------------------------------------------------- //
  public void addOscMap ( OscMap map ) {
    oscMaps.add( map );
  }
  
  // -------------------------------------------------------------------- //
  public void initOscMaps ( OscP5 oscP5 ) {
    for( OscMap Map : oscMaps ) {
      Map.plug( oscP5, this );
    }
  }
  
  // -------------------------------------------------------------------- //
  public void setAll ( int x, int y ) {
    for( Device device : devices ) {
      device.setXY( x, y );
    }
  }
  
  // -------------------------------------------------------------------- //
  public void setAll ( float x, float y, float z ) {
    for( Device device : devices ) {
      device.setXYZ( x, y, z );
    }
  }
  
  // -------------------------------------------------------------------- //
  public void sendAll ( ArduinoControl arduinoControl ) {
    for( Device device : devices ) {
      device.send( arduinoControl );
    }
  }
  
  // -------------------------------------------------------------------- //
  private List<Device> devices = new ArrayList<Device>();
  private List<OscMap> oscMaps = new ArrayList<OscMap>();
}
