//-----Controls-----
// 1-9: Set destination XBee address
// 0:   Set broadcast XBee address
// e:   Echo devices
// i:   Add Arduino devices
// ' ': Toggle device updates
// click: Toggle mouse XY control
//------------------

package com.fieldfx.devicehub;

import com.fieldfx.util.*;
import com.fieldfx.util.serialize.XMLSerializer;

import processing.core.PApplet;
import xbee.*;
import oscP5.*;
import netP5.*;

public class DeviceHub {
  //-----Settings-----
  int            oscListenPort        = 8000;
  int            oscSendPort          = 12000;
  String         oscConnectPattern    = "/server/connect";
  String         oscDisconnectPattern = "/server/disconnect";
  
  //-----Variables-----
  protected ArduinoControl arduinoControl;
  protected DeviceConfig   deviceConfig         = new DeviceConfig();
  protected OscP5          oscP5;
  protected NetAddressList netAddressList       = new NetAddressList();
  protected XMLSerializer  xmlSerializer;
    
  // -------------------------------------------------------------------- //
  public DeviceHub() { }
  // -------------------------------------------------------------------- //
  public void connect ( PApplet parent, String configFileName ) {
    if( xmlSerializer == null ) {
      // Set up Serialization
      xmlSerializer = new XMLSerializer( parent );
      xmlSerializer.registerType( new Pin() );
      xmlSerializer.registerType( new Device() );
      xmlSerializer.registerType( new OscMap() );
      xmlSerializer.registerType( new DeviceConfig() );
    }
    if( arduinoControl == null ) {
      arduinoControl = new ArduinoControl();
      //arduinoControl.setup( parent );
      xmlSerializer.load( configFileName, deviceConfig );
      deviceConfig.initDevices();
      deviceConfig.initOscMaps( oscP5 );
    }
    if( oscP5 == null ) {
      oscP5 = new OscP5( parent, oscListenPort );
    }
  }
  //public void disco
  
  // -------------------------------------------------------------------- //
  private void oscConnect ( String ipAddress ) {
    if( !netAddressList.contains( ipAddress, oscSendPort ) ) {
      netAddressList.add( new NetAddress( ipAddress, oscSendPort ) );
      System.out.print( "### adding " + ipAddress + " to the list." );
    } else {
      System.out.print( "### " + ipAddress + " is already connected." );
    }
    System.out.print( "### currently there are " + netAddressList.list().size() + " remote locations connected." );
  }
  
  // -------------------------------------------------------------------- //
  private void oscDisconnect ( String ipAddress ) {
    if( netAddressList.contains( ipAddress, oscSendPort ) ) {
      netAddressList.remove( ipAddress, oscSendPort );
      System.out.print( "### removing " + ipAddress + " from the list." );
    } else {
      System.out.print( "### " + ipAddress + " is not connected." );
    }
    System.out.print( "### currently there are " + netAddressList.list().size() );
  }
  
  // -------------------------------------------------------------------- //
  private void oscEvent ( OscMessage oscMsg ) {
    if( oscMsg.addrPattern().equals( oscConnectPattern ) ) oscConnect( oscMsg.netAddress().address() );
    else if( oscMsg.addrPattern().equals( oscDisconnectPattern ) ) oscDisconnect( oscMsg.netAddress().address() );
    else //echo back
    oscP5.send( oscMsg, netAddressList );
    
    System.out.println( "received osc msg: " + oscMsg.addrPattern() + " " + oscMsg.typetag() );
  }
  
  // -------------------------------------------------------------------- //
  /*public void createDefaultConfig ( String fileName ) {
    deviceConfig.addDevice( new Device( "xyServo", 1, new int[] { 9, 10 } ) );
    deviceConfig.addDevice( new Device( "xyServo", 1, new int[] { 11, 12 } ) );
    deviceConfig.addDevice( new Device( "xyServo", 2, new int[] { 9, 10 } ) );
    deviceConfig.addOscMap( new OscMap( "laser1", "/multi/1", "setXY" ) );
    deviceConfig.addOscMap( new OscMap( "laser2", "/multi/2", "setXY" ) );
    xmlSerializer.save( "deviceConfig.xml", deviceConfig );
  }*/
  
  // -------------------------------------------------------------------- //
  public Device findDevice ( String deviceName ) {
    if( deviceConfig != null ) return deviceConfig.findDevice( deviceName );
    else return null;
  }
  
  // -------------------------------------------------------------------- //
  public void xBeeEvent ( XBeeReader xbee ) {
    if( arduinoControl != null ) arduinoControl.read();
  }
  
  // -------------------------------------------------------------------- //
  public void update ( PApplet parent ) {
    connect( parent, "deviceConfig.xml" );
    arduinoControl.read();
    
    deviceConfig.sendAll( arduinoControl );
  }
}
