package com.fieldfx.net;

import java.util.ArrayList;
import java.util.List;

import org.json.simple.*;
import org.json.simple.parser.*;
import processing.net.*;
import processing.core.PApplet;

public class NetTracker {
  public class Node {
    public String   uid   = null;
    public String   ip;
    public String   lan_ip;
    public Integer  port;
    
    public void print() {
      System.out.println( uid + " : " + ip + " : " + lan_ip + " : " + port );
    }
  }
  
  private List<Node>  nodes = new ArrayList<Node>();
  private Node        self  = new Node();
  private String      tracker_host;
  private String      tracker_subdir = "";
  
  private Client      c;
  private PApplet     parent;
  private JSONParser  parser = new JSONParser();
  
  // -------------------------------------------------------------------------------- //
  public NetTracker( PApplet parent, String tracker_url, String lan_ip ) {
    this( parent, tracker_url, lan_ip, 16000 + (int)parent.random(0,999) );
  }
  
  // -------------------------------------------------------------------------------- //
  public NetTracker( PApplet parent, String tracker_url, String lan_ip, int port ) {
    this.parent       = parent;
    this.self.port    = port;
    this.self.lan_ip  = lan_ip;
    
    int subdir_start = tracker_url.indexOf('/');
    if( subdir_start > 0 ) {
      this.tracker_host   = tracker_url.substring( 0, subdir_start );
      this.tracker_subdir = tracker_url.substring( subdir_start, tracker_url.length() );
    } else {
      this.tracker_host = tracker_url;
    }
  }
  // -------------------------------------------------------------------------------- //
  public void       setUID( String uid ) { this.self.uid = uid; }
  // -------------------------------------------------------------------------------- //
  public String     getUID()   { return self.uid;    }
  public String     getIP()    { return self.ip;     }
  public String     getLanIP() { return self.lan_ip; }
  public int        getPort()  { return self.port;   }
  // -------------------------------------------------------------------------------- //
  public List<Node> get()      { return nodes; }
  // -------------------------------------------------------------------------------- //
  @SuppressWarnings("unchecked")
  public void beat() {
    if( c == null ) {
      // Connect to server on port 80
      c = new Client(parent, tracker_host, 80); 
  
      JSONObject             obj = new JSONObject();
                             obj.put( "p",   self.port   );
                             obj.put( "ip",  self.lan_ip );
      if( self.uid != null ) obj.put( "uid", self.uid    );
                                     
      String json       = "data=" + obj.toString() + "";
      String user_agent = "Mozilla/5.0 (Windows NT 6.0) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7";

      c.write( "POST " + tracker_subdir + "/ HTTP/1.1\r\n" );
      c.write( "Host: " + tracker_host + "\r\n" );
      c.write( "Connection: keep-alive\r\n" );
      c.write( "Cache-Control: max-age=0\r\n" );
      c.write( "User-Agent: " + user_agent + "\r\n" );
      c.write( "Accept: text/html\r\n" );
      c.write( "Accept-Language: en-US,en;q=0.8\r\n" );
      c.write( "Accept-Charset: ISO-8859-1\r\n" );      
      c.write( "Content-Type: application/x-www-form-urlencoded\r\n" );
      c.write( "Content-Length: " + json.length() + "\r\n" );
      c.write( "\r\n" );
      c.write( json );
    }
  }
  // -------------------------------------------------------------------------------- //
  public void update() {
    if ( c != null && c.available() > 0 ) { // If there's incoming data from the client...
      String   data = c.readString(); // ...then grab it and print it
      //println( data );
      
      int data_start = data.indexOf( "[{" );
      int data_end   = data.indexOf( "}]" );
      if( data_start > 0 && data_end > 0 ) {
        nodes.clear();
        
        try {
          Object    obj = parser.parse( data.substring(data_start, data_end + 2) );
          JSONArray arr = (JSONArray)obj;
          for( Object node : arr ) {
            JSONObject values = (JSONObject)node;
            
            Node n = new Node();
                 n.uid    =  (String)values.get("uid");
                 n.ip     =  (String)values.get("ip");
                 n.lan_ip =  (String)values.get("lan_ip");
                 n.port   =  Integer.parseInt( (String)values.get("port") );
            
            if( self.lan_ip.equals( n.lan_ip ) &&
                self.port.equals( n.port ) ) {
              if( self.uid == null || self.uid.length() == 0 ) {
                self.uid = n.uid;
              }
              self.ip  = n.ip;
            }
            
            nodes.add( n );
          }
        } catch( Exception e ) {
          e.printStackTrace();
        }
      }
      
      c.stop();
      c = null;
    }
  }
  // -------------------------------------------------------------------------------- //
  public void print() {
    for( Node n : nodes ) {
      n.print();
    }
  }
}
