/*
  Caffeine Processing Libraries
  
  (c) copyright
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
 */
 
package com.fieldfx.ui;

import com.fieldfx.math.Vector2;

import processing.core.*;


/**
 * @author Jack Kern
 */
public class UIButton extends UIBox
{
  private   String  caption   = "";
  
  private   boolean isHover         = false;
  private   boolean isPressed       = false;
  private   boolean isToggle        = false;
  private   boolean wasJustClicked  = false;
  
  private   float   hover     = 0.0f;
  private   float   pressed   = 0.0f;
  
  private   PImage  up           = null;
  private   PImage  down         = null;
  
  // ------------------------------------------------------------------------ //
  protected UIButton( UIManager parent, UIContainer container ) {
    super( parent, container );
    setSize( 100, 30 );
  }  
  
  // ------------------------------------------------------------------------ //
  public void setToggle    ( boolean  toggle    ) { this.isToggle  = toggle;    }
  public void setImageUp   ( PImage   up        ) { this.up        = up;        }
  public void setImageDown ( PImage   down      ) { this.down      = down;      }
  

  
  // ------------------------------------------------------------------------ //
  public void update() {
    pressed = PApplet.lerp( pressed, ( isPressed? 1.0f : 0.0f ), 0.75f );
    hover   = PApplet.lerp( hover,   ( isHover?   1.0f : 0.0f ), 0.2f  );    
  }
  
  // ------------------------------------------------------------------------ //
  protected void drawNoImage( UIDrawer d, float width, float height ) {
    d.pushMatrix();
      d.rotate( PI * 0.5f );
      d.translate( 0, -width );
      d.boxType( UIDrawer.BUTTON );
      d.box( 0, 0, height, width );
    d.popMatrix();  
  }
  
  // ------------------------------------------------------------------------ //
  protected void drawImage( UIDrawer d, float width, float height, PImage img ) {
    d.image( img, 0, 0, width, height );
  }  
  
  // ------------------------------------------------------------------------ //
  public void drawButton( UIDrawer d, float hover, float pressed ) {
    float textSize  = 10.f;
    float textWidth = d.textWidth( caption );
    float width     = size.x;
    float height    = size.y;
    
    if( up == null )
    {
      width  = PApplet.max( textWidth + textSize, width  );    
      height = PApplet.max( textSize * 2.0f,       height );
    }
    
    d.pushMatrix();
      d.translate ( 3 * pressed, 4 * pressed );
      
      if( up   != null )
      {
        if ( down == null )  drawImage( d, width, height, up );
        else                 drawImage( d, width, height, pressed < 0.5f ? up : down );
      }
      else
      {
        drawNoImage( d, width, height );
        
        float x = width  * 0.5f - textWidth * 0.5f;
        float y = height * 0.5f + textSize  * 0.5f;
        
        d.textType( UIDrawer.LABEL );
        d.text( caption, x, y );
      }
      
    d.popMatrix();
  }
  // ------------------------------------------------------------------------ //
  public void draw( UIDrawer d )
  {
    transform  ( d );
    drawButton ( d, hover, pressed );   
  }
  // ------------------------------------------------------------------------ //
  public void   setCaption  ( String caption ) { this.caption = caption; }
  public String getCaption  (                ) { return caption;    }
  // ------------------------------------------------------------------------ //
  public String getTypeName (                ) { return "UIButton";    } 
 
  // ------------------------------------------------------------------------ //
  
  protected boolean mousePressed()
  {
    Vector2 clientMouse = getClientMouseOnly();
    if( clientMouse != null && getApp().mouseButton == LEFT )
    {
      if( isToggle ) isPressed      = !isPressed;
      else           isPressed      = true;
                     wasJustClicked = true;
      return true;
    }
    
    return false;
  }
  protected boolean mouseReleased ()
  {
    boolean wasHandled = false;
    
    // Send a clicked message only when we just let go inside the button
    if( wasJustClicked )
    {
      //@TODO
      wasHandled = true;
    }
    
    if( !isToggle ) isPressed      = false;
                    wasJustClicked = false;

    return wasHandled;
  }  
  protected boolean mouseMoved    ()
  {
    Vector2 clientMouse = getClientMouseOnly();
    isHover = false;
    
    if( clientMouse != null ) {
      isHover = true;
      
      // call the hover command if it exists
      //if( commandHover != null ) {
        //@TODO
      //}
        
      return true;
    }
    
    return false;
  }
}