/*
  Caffeine Processing Libraries
  
  (c) copyright
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
 */
 
package com.fieldfx.ui;

import com.fieldfx.math.Vector2;


/**
 * @author Jack Kern
 */
public class UISlider extends UIBox
{
  protected float   percent;
  protected float   viewPercent;
  protected float   shade;
  
  // -------------------------------------------------------------------------------------------------------------- //
  public    UISlider     setPos              ( float x, float y )   { return (UISlider)super.setPos(x,y); }
  public    UISlider     setPos              ( Vector2 pos )        { return (UISlider)super.setPos(pos); }  
  public    UISlider     setHeight           ( float h )            { return (UISlider)super.setHeight(h); }
  public    UISlider     setWidth            ( float w )            { return (UISlider)super.setWidth(w); }  
  public    UISlider     setSize             ( float w, float h )   { return (UISlider)super.setSize(w,h); }   
  public    UISlider     setSize             ( Vector2 size )       { return (UISlider)super.setSize(size); }  
  public    UISlider     setMinSize          ( Vector2 size )       { return (UISlider)super.setMinSize(size); }  
  public    UISlider     setMinSize          ( float w, float h )   { return (UISlider)super.setMinSize(w,h); }  
  public    UISlider     setRot              ( float r )            { return (UISlider)super.setRot(r); }
  public    UISlider     setScale            ( float s )            { return (UISlider)super.setScale(s); }  
  public    UISlider     show                ( )                    { return (UISlider)super.show(); }
  public    UISlider     hide                ( )                    { return (UISlider)super.hide(); }  
  public    UISlider     setFocus            ( )                    { return (UISlider)super.setFocus(); }   
  public    UISlider     setFocus            ( boolean bSet )       { return (UISlider)super.setFocus( bSet ); }     
  public    UISlider     bringToFront        ( )                    { return (UISlider)super.bringToFront(); }
  public    UISlider     sendToBack          ( )                    { return (UISlider)super.sendToBack(); }
  
  public    UISlider     setViewPercent      ( float viewPercent ) { this.viewPercent = viewPercent; return this; } 
  public    UISlider     setSlider           ( float percent     ) { this.percent     = percent;     return this; }
  public    UISlider     setShade            ( float shade       ) { this.shade       = shade;       return this; }  

  // ------------------------------------------------------------------------ //
  public    float        getViewPercent      ( ) { return viewPercent; } 
  public    float        getSlider           ( ) { return percent; }
  public    float        getShade            ( ) { return shade; }  
  public    String       getTypeName         ( ) { return "UISlider"; } 

  
  // ------------------------------------------------------------------------ //
  protected UISlider( UIManager parent, UIContainer container ) {
    super( parent, container );
    setSize( 20, 200 );
  }
  
  // ------------------------------------------------------------------------ //
  public void update() { }
  
  public void drawSlider( UIDrawer d, float percent, float fPos, float shade ) {
    float width      = size.x;
    float length     = size.y;
    float dragLength = length * percent;  
    
    if( dragLength < 30  )    dragLength = 30;
    if( dragLength > length ) dragLength = length;
   
    d.pushMatrix();
      float x = width * 0.5f;
      
      d.boxType( UIDrawer.SLIDEAREA );
      d.box  ( 0, 0, width, length );
      
      d.boxType( UIDrawer.BUTTON );
      
      float fYS = fPos * (length - dragLength);

      d.box   ( 0, fYS, width, dragLength );
      
      d.notch ( 3, fYS + dragLength / 2 - 3, width - 6 );
      d.notch ( 3, fYS + dragLength / 2,     width - 6 );
      d.notch ( 3, fYS + dragLength / 2 + 3, width - 6 );
      
    d.popMatrix();
  }
  
  public void draw( UIDrawer d ) {
    transform  ( d );
    drawSlider ( d, viewPercent, percent, shade );
  }

  protected boolean mouseMoved()  { return false; }
  
  protected boolean mouseDragged() {
    if( !hasFocus() ) return false;
    
    Vector2 clientMouse = getClientMouse();

    percent = parent.getApp().map( clientMouse.y, 30, size.y, 0.0f, 1.0f);
    percent = percent < 0 ? 0 : percent;
    percent = percent > 1 ? 1 : percent;

    return true;
  }
  
  protected boolean mouseScrolled () { return false; }    


}
