/** Serialization.pde
 *  Jack Kern - http://www.jackkern.com
 *  
 *  An example demonstrating the advantages of XMLSerialization, including a 
 *  basic implementation.
 */
 
 int type = 2;

// ------------------------------------------------------------ // 
void setup() {
  
  // First create the serializer object
  Serializer store = null;
  
  if      ( type == 1 ) store = new XMLSerializer( this );
  else if ( type == 2 ) store = new BinarySerializer();
  
  // Next register your serializable types so they
  //  can be instantiated on load. (Only required for list-serialization support)
  store.registerType( new Data() );
  store.registerType( new PVectorData() );
  
  // Finally let's test saving out some data, and loading it back in.
  Object obj = testSaving( store );
  println("------------------------------------------------------------");
  testLoading( obj, store );

  // Fin.
  exit();  
}

// ------------------------------------------------------------ // 
void testLoading( Object obj, Serializer s ) {
  // First we create our data object, then use the serializer to load it, and finally
  //  we print it to the debug console to verify success!
  Data data = new Data();
  
  if( type == 1 ) {
    XMLSerializer store = (XMLSerializer)s;
                  store.load( "data.xml", data );
  }
  else if( type == 2 ) {
    
    byte[]           bytes = (byte[])obj;
    BinarySerializer store = (BinarySerializer)s;
                     store.load( bytes, data );
  }
  
  data.print();
}

// ------------------------------------------------------------ // 
Object testSaving( Serializer s ) {
  // Notice it is possible to easily save a tree structure because "Data" types can
  //  store "Data" objects as thier own children.
  Data  data =             new Data( 1, 2.0f, "root",    false, 10.0f,  20.0f, 30.0f );
        data.children.add( new Data( 2, 1.0f, "child 1", true,   1.2f,   3.4f,  5.6f ) );
        data.children.add( new Data( 3, 1.1f, "child 2", false, 40.0f, -10.1f, 62.0f ) );
        data.print();

  if( type == 1 ) {
    XMLSerializer store = (XMLSerializer)s;
                  store.save( "data.xml", data );
  }                
  else if( type == 2 ) {
    BinarySerializer store = (BinarySerializer)s;
    byte[]           bytes = store.save( data );
    return           bytes;
  }

  return null;
}


