/*
  Caffeine Processing Libraries
  
  (c) copyright
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
 */
 
package com.fieldfx.ui;

// -------------------------------------------------------------------------------------------------------------- //
/** This interface defines the contract that any container of UIElements must implement. See {@UIView} for 
  * further details.
  * @author Jack Kern
  * @version 1.0 */
public interface UIContainer
{

// -------------------------------------------------------------------------------------------------------------- //
/** Returns the {@link UIView} for this container. The view applies its transformation on any child elements.
  * @return    The view for this container. */
  public  UIView     getView     ();

// -------------------------------------------------------------------------------------------------------------- //
/** Adds a child element to this {@link UIContainer}, and returns the added element. This function removes
  * the responsibility from {@link UIManager} to update the element, unless the {@link UIContainer} is the 
  * {@link UIManager}. It then becomes this {@link UIContainer}'s responsibility to update and provide 
  * event notifiations.
  *
  * @param     elm   The element to be added as a child of this {@link UIContainer}.
  * @return          The element that was added as a child. */
  public  UIElement  addChild    ( UIElement elm );

// -------------------------------------------------------------------------------------------------------------- //
/** Removes a child element from this {@link UIContainer}, and returns true if the element was found and 
  * could be successfully removed. This function removes the responsibility from this {@link UIContainer} to update the element
  *
  * @param    elm   The element to be removed from the list of children in this {@link UIContainer}
  * @return         True if the element could be removed, false otherwise. */
  public  boolean    removeChild ( UIElement elm );

// -------------------------------------------------------------------------------------------------------------- //
/** Brings the specified child element to the front of this container's list of elements.
  * @param    elm   The element to be moved to the front of the list of visible child elements.
  * @return         Returns the element that was brought to the front.*/
  public  UIElement  bringToFront( UIElement elm );

// -------------------------------------------------------------------------------------------------------------- //
/** Sends the specified child element to the back of this container's list of elements.
  * @param    elm   The element to be moved to the back of the list of visible child elements.
  * @return         Returns the element sent to the back.*/
  public  UIElement  sendToBack  ( UIElement elm );
}