/*
  Caffeine Processing Libraries
  
  (c) copyright
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
 */
 
package com.fieldfx.ui;

import processing.core.PApplet;
import processing.core.PGraphics;
import processing.core.PImage;
import processing.core.PConstants;
import processing.core.PFont;

public class UIDrawerBasic implements UIDrawer
{
  private int       boxType   = 0;
  private int       textType  = 0;
  
  private int       arrowSize = 5;
  
  private PGraphics g;
  private PImage    closeButton  = null;
  private PImage[]  minButton    = new PImage[2];
  private PImage[]  pinnedButton = new PImage[2];
  
  private PFont     font;
  
  public UIDrawerBasic( PApplet p )
  {
    g = p.g;
    
    font = p.createFont( "Arial", 72 );
    g.textFont( font, 18.0f );
    
    buildCloseButton(p);
    buildMinButton(p);
    buildPinnedButton(p);
  }

  public PGraphics  graphics() { return g; }
  
  public PImage     getCloseButton()   { return closeButton; }
  public PImage[]   getMinButton()     { return minButton; }
  public PImage[]   getPinnedButton()  { return pinnedButton; }  

  public void       translate   ( float x, float y )  { g.translate(x,y); }
  public void       rotate      ( float angle )       { g.rotate(angle);  }
  public void       scale       ( float scale )       { g.scale(scale);   }
  public void       pushMatrix  ()                    { g.pushMatrix();   }
  public void       popMatrix   ()                    { g.popMatrix();    }

  public float      textWidth   ( String text )       { return g.textWidth( text ); }
  public void       boxType     ( int type )          { boxType = type; }
  
  public void       box         ( float x, float y, float width, float height )
  {
    g.pushStyle();
    
    if( boxType == BOX ) {
      
      g.fill(200,128);
      g.noStroke();
      g.rect(x,y,width,height); 
      
    } else if( boxType == BUTTON ) {
      
      g.fill(200);
      g.noStroke();
      g.rect(x,y,width,height);    
      
      g.stroke(255);
      g.line(x,y,x,y+height);
      g.line(x,y,x+width,y);
      
      g.stroke(0);
      g.line(x+1,y+height,x+width,y+height);
      g.line(x+width,y+1,x+width,y+height);
      
    } else if( boxType == SLIDEAREA ) {
      
      g.fill(230);
      g.noStroke();
      g.rect(x,y,width,height);
      
    } else if( boxType == HIGHLIGHT ) {
      
      g.noStroke();
      g.fill(200,200,0);
      g.rect(x,y,width,height);
      
    } else if( boxType == SHADOW ) {
      //new Vector2( 20, 20 )
      
      g.noStroke();
      g.fill(0,32);
      g.rect(10+x,10+y,width,height);
      
    } else if( boxType == EDITBOX ) {
      
      g.stroke(0);
      g.fill(255);
      g.rect(x,y,width,height);
      
    } else if( boxType == TITLEBAR ) {
      
      g.noStroke();
      g.fill(50,150,180);
      
      float r = height;
      g.arc      (       r/2, r/2,        r,       r,    PConstants.PI,   PConstants.PI*1.5f );
      g.arc      ( width-r/2, r/2,        r,       r,   -PConstants.PI/2,      0.0f );
      g.rect     (       r/2,   0,  width-r, 0.5f*r  );
      g.rect     (         0, r/2,    width, 0.5f*r  );

    } else {
      
      g.noFill();
      g.stroke(255);
      g.rect(x,y,width,height);
      
    }
    
    g.popStyle();
  }

  public void       image       ( PImage img ) { image( img, 0, 0, img.width, img.height ); }
  public void       image       ( PImage img, float x, float y, float width, float height )
  {
    g.image(img,x,y,width,height);
  }
    
  public void knob ( float x, float y, float diameter  )
  {
    g.pushStyle();
      g.fill(230);
      g.stroke(0);
      g.pushMatrix();
        g.translate( diameter / 2.f, diameter / 2.f );
        g.ellipse( x, y, diameter, diameter );
      g.popMatrix();
    g.popStyle();
  }
  
  public void notch ( float x, float y, float w     )
  {
    g.pushStyle();
      g.strokeWeight( 1.0f );
      int low  = 100;
      int high = 250;
      g.stroke( low  ); g.line( x,     y,     x + w - 1, y     );
      g.stroke( low  ); g.line( x,     y,     x,         y + 1 );
      g.stroke( high ); g.line( x + w, y,     x + w,     y + 1 );
      g.stroke( high ); g.line( x + 1, y + 1, x + w,     y + 1 );
    g.popStyle();
  }
  
  public void arrow ( float x, float y, int direction )
  {
    g.pushStyle();
      g.noStroke ( );
      g.fill     ( 64 );
      
      g.pushMatrix();
        g.translate( x, y );
      
        if( direction == 1 ) g.scale(1,-1);
        
        g.triangle ( 0,0,-arrowSize,arrowSize,arrowSize,arrowSize );
      g.popMatrix();  
    g.popStyle();
  }


  public void       textType    ( int type )                          { textType = type; }
  public void       text        ( String text, float x, float y )
  {
    g.pushStyle();
          
      if( textType == EDITBOX || textType == LABEL ) g.fill(255);
      else                                           g.fill(0);
      g.text( text, (int)x+1, (int)y+1 );

      if( textType == EDITBOX || textType == LABEL ) g.fill(0);
      else                                           g.fill(255);
      g.text( text, (int)x, (int)y );
      
    g.popStyle();
  }
  
  
  public void       beginBorder ( float borderWidth, float depth, int nesting )
  {
  }
  public void       corner      ( float x, float y, float radius )
  {
  }
  public void       endBorder   ( boolean close )
  {
  }
  
  
  private final int a = 2;
  private final int b = 6;
  private final int c = 8;
  private final int d = 10;
  private final int e = 14;
  
  public void buildCloseButton( PApplet p )
  {
    PGraphics gfx;
    
    gfx = beginBuildButton( p );
      gfx.line(a,a,e,e);
      gfx.line(e,a,a,e);
    gfx.endDraw();

    closeButton = gfx;
  }
  
  public void buildMinButton( PApplet p )
  {
    PGraphics gfx;
    
    gfx = beginBuildButton( p );
      gfx.line(a,c,d,c);
    gfx.endDraw();
    minButton[0] = gfx;
    
    gfx = beginBuildButton( p );
      gfx.line(b,c,d,c);
    gfx.endDraw();
    minButton[1] = gfx;
  }
  
  public void buildPinnedButton( PApplet p )
  {
    PGraphics gfx;
    
    gfx = beginBuildButton(p);
      gfx.line(c,a,c,e);
    gfx.endDraw();
    pinnedButton[0] = gfx;
    
    gfx = beginBuildButton(p);
      gfx.line(c,b,c,d);
    gfx.endDraw();
    pinnedButton[1] = gfx;  
  }

  private PGraphics beginBuildButton( PApplet p )
  {
    PGraphics gfx = p.createGraphics(16,16,PGraphics.JAVA2D);  
  
    gfx.beginDraw();    
    gfx.stroke(255);
    gfx.strokeWeight(2);
    
    return gfx;
  }

}
