/*
  Caffeine Processing Libraries
  
  (c) copyright
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
 */
 
package com.fieldfx.ui;

import com.fieldfx.math.Vector2;


// -------------------------------------------------------------------------------------------------------------- //
/** UIFrame is an extention of UIBox that provides support for child elements.
 * @author Jack Kern
 */
public class UIFrame extends UIBox implements UIContainer
{ 
  // -------------------------------------------------------------------------------------------------------------- //
  protected UIView      childView     = null;
  protected float       borderSize    = 4.0f;
  
  
  // ------------------------------------------------------------------------ //
  protected UIFrame( UIManager parent, UIContainer container ) {
    super( parent, container );

    childView = new UIView( parent, getWidth(), getHeight() );  // Create the child view
    childView.setParentTransform( this );                       // Connect the transform
    
    updateView();
  }

  // -------------------------------------------------------------------------------------------------------------- //
  public    UIFrame     setPos          ( float x, float y )   { return (UIFrame)super.setPos(x,y); }
  public    UIFrame     setPos          ( Vector2 pos )        { return (UIFrame)super.setPos(pos); }  
  public    UIFrame     setHeight       ( float h )            { return (UIFrame)super.setHeight(h); }
  public    UIFrame     setWidth        ( float w )            { return (UIFrame)super.setWidth(w); }  
  public    UIFrame     setSize         ( float w, float h )   { UIFrame f = (UIFrame)super.setSize(w,h); updateView(); return f; } 
  public    UIFrame     setSize         ( Vector2 s )          { UIFrame f = (UIFrame)super.setSize(s);   updateView(); return f; }  
  public    UIFrame     setMinSize      ( Vector2 s )          { return (UIFrame)super.setMinSize(s); }  
  public    UIFrame     setMinSize      ( float w, float h )   { return (UIFrame)super.setMinSize(w,h); }  
  public    UIFrame     setRot          ( float r )            { return (UIFrame)super.setRot(r); }
  public    UIFrame     setScale        ( float s )            { return (UIFrame)super.setScale(s); }  
  public    UIFrame     show            ( )                    { return (UIFrame)super.show(); }
  public    UIFrame     hide            ( )                    { return (UIFrame)super.hide(); }  
  public    UIFrame     setFocus        ( )                    { return (UIFrame)super.setFocus(); }   
  public    UIFrame     setFocus        ( boolean bSet )       { return (UIFrame)super.setFocus( bSet ); }     
  public    UIFrame     bringToFront    ( )                    { return (UIFrame)super.bringToFront(); }
  public    UIFrame     sendToBack      ( )                    { return (UIFrame)super.sendToBack(); }
  public    UIFrame     setBorderSize   ( float size )         { borderSize = size; return this; }

  
  // ------------------------------------------------------------------------ //
  public    float       getBorderSize   ( )             { return borderSize; }
  public    float       getClientWidth  ( )             { return childView.getWidth(); }
  public    float       getClientHeight ( )             { return childView.getHeight(); }
  public    Vector2     getClientSize   ( )             { return childView.getSize(); }
  public    UIView      getView         ( )             { return childView; }
  
  // ------------------------------------------------------------------------ //
  public    String      getTypeName()               { return "UIFrame"; }
  
  public    void        update()                    {  super.update();   childView.update(); }
  public    void        onResize()                  {  super.onResize(); }
  
  // ------------------------------------------------------------------------ //
  public   UIElement  bringToFront ( UIElement elm ) { return childView.bringToFront( elm ); }
  public   UIElement  sendToBack   ( UIElement elm ) { return childView.sendToBack( elm ); }
  public   UIElement  addChild     ( UIElement elm ) { return childView.addChild(elm);    }
  public   boolean    removeChild  ( UIElement elm ) { return childView.removeChild(elm); }
  // ------------------------------------------------------------------------ //
  public void draw( UIDrawer d ) {
    updateView();
    transform( d );
    drawFrame( d );
    childView.draw( d );
  }
  // ------------------------------------------------------------------------ //
  protected void updateView() {
    if( childView == null ) return;
    childView.setPos  ( borderSize, borderSize );
    childView.setSize ( size.sub    ( borderSize*2, borderSize*2 ) );
  }
  // ------------------------------------------------------------------------ //
  protected boolean inputEvent( int eventIndex ) {
    return childView.inputEvent( eventIndex ) ? true : super.inputEvent( eventIndex );
  }
  // ------------------------------------------------------------------------ //
  protected void drawFrame( UIDrawer d )
  {
    drawShadow( d );
    drawBox( d );
  }
}
