/*
  Caffeine Processing Libraries
  
  (c) copyright
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
 */
 
package com.fieldfx.ui;

import com.fieldfx.math.Vector2;

/**
 * @author Jack Kern
 */
public class UIKnob extends UIBox
{
  private float   angle   = 0;
  private boolean isGrabbed = false;

  // ------------------------------------------------------------------------ //
  protected UIKnob( UIManager parent, UIContainer container ) {
    super( parent, container );
    setRadius( 50.f );
  }
  
  // -------------------------------------------------------------------------------------------------------------- //
  public    UIKnob        setPos              ( float x, float y )   { return (UIKnob)super.setPos(x,y); }
  public    UIKnob        setPos              ( Vector2 pos )        { return (UIKnob)super.setPos(pos); }  
  public    UIKnob        setHeight           ( float h )            { return (UIKnob)super.setHeight(h); }
  public    UIKnob        setWidth            ( float w )            { return (UIKnob)super.setWidth(w); }  
  public    UIKnob        setSize             ( float w, float h )   { return (UIKnob)super.setSize(w,h); }   
  public    UIKnob        setSize             ( Vector2 size )       { return (UIKnob)super.setSize(size); }  
  public    UIKnob        setMinSize          ( Vector2 size )       { return (UIKnob)super.setMinSize(size); }  
  public    UIKnob        setMinSize          ( float w, float h )   { return (UIKnob)super.setMinSize(w,h); }  
  public    UIKnob        setRot              ( float r )            { return (UIKnob)super.setRot(r); }
  public    UIKnob        setScale            ( float s )            { return (UIKnob)super.setScale(s); }  
  public    UIKnob        show                ( )                    { return (UIKnob)super.show(); }
  public    UIKnob        hide                ( )                    { return (UIKnob)super.hide(); }  
  public    UIKnob        setFocus            ( )                    { return (UIKnob)super.setFocus(); }   
  public    UIKnob        setFocus            ( boolean bSet )       { return (UIKnob)super.setFocus( bSet ); }     
  public    UIKnob        bringToFront        ( )                    { return (UIKnob)super.bringToFront(); }
  public    UIKnob        sendToBack          ( )                    { return (UIKnob)super.sendToBack(); }  
  public    UIKnob        setAngle            ( float angle )        { this.angle = angle; return this; }
  public    UIKnob        setRadius           ( float size )         { getSize().set( size*2 ); return this; }
  
  // ------------------------------------------------------------------------ //
  public    float         getAngle            ()                     { return angle; }
  public    float         getPercent          ()                     { return angle / (2.f * PI); }
  public    float         getRadius           ()                     { return getSize().x/2; }
  public    String        getTypeName         ()                     { return "UIKnob"; }   
  
  // ------------------------------------------------------------------------ //
  public void drawKnob( UIDrawer d, float fRot )
  {
    float fRad = getRadius();
    
    d.pushMatrix();
      d.knob      ( 0, 0, fRad );   
      d.translate ( fRad*0.5f, fRad*0.5f );
      d.rotate    ( fRot );
      d.arrow     ( 0, -fRad * 0.5f + 4, 0 );
    d.popMatrix();
  }
  // ------------------------------------------------------------------------ //
  public void draw( UIDrawer d ) {
    transform( d );
    drawKnob( d, angle );
  }
  // ------------------------------------------------------------------------ //
  protected boolean mousePressed() {
    if( getApp().mouseButton == LEFT && getClientMouseOnly() != null ) {
      setFocus();
      isGrabbed = true;
      return true;
    }
    return false;
  }
  // ------------------------------------------------------------------------ //
  protected boolean mouseReleased() {
    isGrabbed = false;
    return false;
  }
  // ------------------------------------------------------------------------ //
  protected boolean mouseDragged() {
    if( hasFocus() && isGrabbed ) {
      Vector2 vClientMouse = getClientMouse();
              vClientMouse.dec( getSize().mul( 0.5f ) );
      angle = vClientMouse.ang() + PI*0.5f;
      
      return true;
    }
    return false;
  }
}