/*
  Caffeine Processing Libraries
  
  (c) copyright
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
 */
 
package com.fieldfx.ui;

import processing.core.*;

import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.lang.String;

/**
 * @author Jack Kern
 */
public class UIManager
{
  public final String VERSION = "0.3.0";

  //---------------------------------------------------------------------------------//
  private PApplet    parent   = null;
  private UIFactory  factory  = null;
  private UIView     view     = null;
  private UIDrawer   drawer   = null;
  private UIElement  focus    = null;

  //---------------------------------------------------------------------------------//
  public UIManager( PApplet parent ) {
    this( parent, new UIDrawerBasic(parent) );
  }
  //---------------------------------------------------------------------------------//
  public UIManager( PApplet parent, UIDrawer drawer ) {
    this.parent = parent;
    this.parent.registerPre        ( this );
    this.parent.registerMouseEvent ( this );
    this.parent.registerKeyEvent   ( this );
    
    view         = new UIView      ( this, parent.width, parent.height );
    factory      = new UIFactory   ( this, view );
    this.drawer  = drawer;
    
    view.enableDrag();
    view.enableZoom();
    
    // Add the mouse wheel listener
    parent.addMouseWheelListener( new java.awt.event.MouseWheelListener() { 
      public void mouseWheelMoved( java.awt.event.MouseWheelEvent evt ) { 
        mouseScrolled( evt.getWheelRotation() ); } } );      
  }
  
  //---------------------------------------------------------------------------------//
  public PApplet          getApp()        { return parent;  }
  public UIFactory        getUIFactory()  { return factory; }
  public UIView           getView()       { return view;    }
  public UIDrawer         getDrawer()     { return drawer; }
  //---------------------------------------------------------------------------------//
  public void setFocus( UIElement elem, boolean bSet ){
    if( bSet ) {
      focus = elem;
      view.bringToFront( elem );
    }
  }
  //---------------------------------------------------------------------------------//
  public boolean   hasFocus   ( )                { return focus != null;  }
  public boolean   hasFocus   ( UIElement elem ) { return elem == focus; }
  public UIElement getFocus   ( )                { return focus;         }
  public UIElement clearFocus ( )                { return focus = null;  }

  public String    version    ( )                { return VERSION; }

  //---------------------------------------------------------------------------------//
  /// This call draws all of the user interface. This call must be in your
  ///  draw function to see any UI.
  public void draw() {
    view.draw( drawer );
  }
  public void pre() {
    view.update();
  }

  //---------------------------------------------------------------------------------//
  public void keyEvent( KeyEvent event ) {
    view.inputEvent( event.getID() );
  }
  //---------------------------------------------------------------------------------//
  public void mouseScrolled( int delta ) {
    view.mouseScrolled( delta ); 
  }
  //---------------------------------------------------------------------------------//
  public void mouseEvent( MouseEvent event ) {
    int id = event.getID();
    if( !view.inputEvent( id ) && id == MouseEvent.MOUSE_PRESSED )
      clearFocus();      // Since we didn't hit anything, clear our focus!
  }
}
