
 
package com.fieldfx.ui;

import java.lang.String;

import com.fieldfx.math.Vector2;

/**
 * @author Jack Kern
 */
public class UIWindow extends UIFrame {
  
  private String   caption            = "Window";
  private float    titleHeight        = 18.0f;
  private float    worldTitleHeight   = 0.f;
  private int      titleColor         = 0xFF404040;
  private int      titleCaptionColor  = 0xFFFFFFFF;
  
  private Vector2  regularSize       = new Vector2();
  
  //---------------------------------------------------------------------------//
  protected UIWindow( UIManager parent, UIContainer container ) {
    super( parent, container );
    setSize( 640, 480 );
  }
  
  // -------------------------------------------------------------------------------------------------------------- //
  public    UIWindow     setPos                ( float x, float y )   { return (UIWindow)super.setPos(x,y); }
  public    UIWindow     setPos                ( Vector2 pos )        { return (UIWindow)super.setPos(pos); }  
  public    UIWindow     setHeight             ( float h )            { return (UIWindow)super.setHeight(h); }
  public    UIWindow     setWidth              ( float w )            { return (UIWindow)super.setWidth(w); }  
  public    UIWindow     setSize               ( float w, float h )   { return (UIWindow)super.setSize(w,h); }   
  public    UIWindow     setSize               ( Vector2 size )       { return (UIWindow)super.setSize(size); }  
  public    UIWindow     setMinSize            ( Vector2 size )       { return (UIWindow)super.setMinSize(size); }  
  public    UIWindow     setMinSize            ( float w, float h )   { return (UIWindow)super.setMinSize(w,h); }  
  public    UIWindow     setRot                ( float r )            { return (UIWindow)super.setRot(r); }
  public    UIWindow     setScale              ( float s )            { return (UIWindow)super.setScale(s); }  
  public    UIWindow     show                  ( )                    { return (UIWindow)super.show(); }
  public    UIWindow     hide                  ( )                    { return (UIWindow)super.hide(); }  
  public    UIWindow     setFocus              ( )                    { return (UIWindow)super.setFocus(); }   
  public    UIWindow     setFocus              ( boolean bSet )       { return (UIWindow)super.setFocus( bSet ); }     
  public    UIWindow     bringToFront          ( )                    { return (UIWindow)super.bringToFront(); }
  public    UIWindow     sendToBack            ( )                    { return (UIWindow)super.sendToBack(); }
  public    UIWindow     setBorderSize         ( float size )         { return (UIWindow)super.setBorderSize( size ); }  
  
  public    UIWindow     setCaption            ( String caption )     { this.caption  = caption;  return this; }
  
  // -------------------------------------------------------------------------------------------------------------- //
  public    String       getCaption            ()                    { return caption; }

  //---------------------------------------------------------------------------//
  public void    move               ( Vector2 velocity )                                { super.move( velocity ); }
  //---------------------------------------------------------------------------//
  public int     searchMaxCharLength( String strText, float fWithinSize )               { return searchMaxCharLength( strText, 0, fWithinSize ); } 
  public int     searchMaxCharLength( String strText, int nColLeft, float fWithinSize ) {
    int nLineWidth = strText.length();
    if( nLineWidth >= 2 ) {
      int    nCol       = nLineWidth / 2;
      String strLeft    = strText.substring( 0,nCol );
      float  fLeftWidth = getApp().textWidth( strLeft );
      
      if( fWithinSize <= fLeftWidth )
        return searchMaxCharLength( strLeft, nColLeft, (int)fWithinSize );
        
      return searchMaxCharLength( strText.substring( nCol ), nColLeft + strLeft.length(), (int)(fWithinSize - fLeftWidth) );
    }
    return nColLeft;
  }

  //---------------------------------------------------------------------------//
  protected void updateView() {
    if( getView() != null ) {
      super.updateView();

      Vector2 sizeview = getView().getSize();
              sizeview.y -= worldTitleHeight;
      getView().setSize ( sizeview );
    }  
  }
  //---------------------------------------------------------------------------//
  public void drawWindow( UIDrawer d ) {
    
    updateView( );
    
    transform( d ); // Apply client transform
    

    drawShadow( d );
    drawTitle( d );  
    
    d.pushMatrix();
        d.translate( 0, worldTitleHeight  );    
        size.y -= worldTitleHeight;
          drawBox( d );  
        size.y += worldTitleHeight;
    d.popMatrix();

  }
  
  
  //---------------------------------------------------------------------------//
  public void draw( UIDrawer d ) {
    drawWindow( d );
    d.translate( 0, worldTitleHeight  );    
    getView().draw( d );
  }

  //---------------------------------------------------------------------------//
  protected void drawTitle( UIDrawer d ) {
    
    Vector2 vTitle      = clientToWorld( new Vector2( titleHeight - 4, titleHeight ), UITRANS_SCALE );
    worldTitleHeight    = vTitle.y;
    
    d.pushMatrix();
    
      d.boxType( UIDrawer.TITLEBAR );
      d.box( 0,0, size.x, vTitle.y );
      
      // Make sure that this window actually has a caption to draw
      if( caption != null && !caption.equals("") )
      {
        String strSubCaption = caption;
        
        Vector2 winScreenSize    = worldToClient( size, UITRANS_SCALE );
        float   titleAreaWidth   = winScreenSize.x - 10; // Padding
        
        if( d.textWidth( caption ) > titleAreaWidth ) {
          int nChars    = searchMaxCharLength( caption, titleAreaWidth );
          strSubCaption = caption.substring( 0,nChars );
        }
        
        d.translate( 10, vTitle.x );
        d.scale( worldTitleHeight / titleHeight );
        
        d.textType( UIDrawer.TITLE );
        d.text( strSubCaption, 0, 0 );
      }
      
    d.popMatrix();
  }

  //---------------------------------------------------------------------------//
  public void update()
  {
    super.update();

    //drawSize.set( size );
    
    //if( caption != null && caption.length() > 0 )
      //drawSize.x += getDrawer().textWidth( caption );
      
    //drawSize.set( clientToWorld( drawSize, UITRANS_SCALE ) );

  }

  // -------------------------------------------------------------------------------------------------------------- //
  public String getTypeName() { return "UIWindow"; }
}
